package eu.etaxonomy.taxeditor.prototype1.view;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.prototype1.Activator;
import eu.etaxonomy.taxeditor.prototype1.controller.OpenNameEditorAction;

public class NameListView extends ViewPart {
	private Table table;
	private Table rnTable;	
	private Composite nameListComposite = null;
	private Composite recentNamesComposite = null;
	private Composite searchComposite = null;	
	private Composite top = null;
	private ExpandBar expandBar = null;
	private TableViewer viewer;
	private TableViewer rnViewer;
	private ExpandItem nameListExpandItem;
	private ExpandItem recentNamesExpandItem;
	private ExpandItem searchExpandItem;	
	public static final String ID = "eu.etaxonomy.mvctest.view.NameListView"; //$NON-NLS-1$
		
	/**
	 * Create contents of the view part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;
		top = new Composite(parent, SWT.NONE);
		createExpandBar();
		top.setLayout(gridLayout);
	}

	/**
	 * This method initializes expandBar
	 * 
	 */
	private void createExpandBar() {
		expandBar = new ExpandBar(top, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		expandBar.setLayoutData(gridData);
		createNameListComposite();
		createRecentNamesComposite();
		createSearchComposite();
		
		createActions();
		initializeToolBar();
		initializeMenu();		
		
	}
	
	private void createNameListComposite() {
		nameListExpandItem = new ExpandItem(expandBar, SWT.NONE, 0);
		nameListExpandItem.setHeight(500);
		
		nameListExpandItem.setText("Name List");
		nameListExpandItem.setExpanded(true);
//		Image image = FileArrangerImages.getImageRegistry().get(
//				IFileArrangerImages.IMG_HOME);
//		nameListExpandItem.setImage(image);
		
		nameListComposite = new Composite(expandBar, SWT.NONE);
//		nameListExpandItem.setHeight(nameListComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		GridLayout layout = new GridLayout();
		layout.verticalSpacing = 0;
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.horizontalSpacing = 0;
//		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
//		layout.verticalSpacing = 5;
		nameListComposite.setLayout(layout);
		
		viewer = new TableViewer(nameListComposite, SWT.H_SCROLL
				| SWT.V_SCROLL);
		table = viewer.getTable();
		final GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true);
//		gd_table.heightHint = 238;
//		gd_table.widthHint = 443;
		table.setLayoutData(gd_table);
				
		viewer.addDoubleClickListener(new IDoubleClickListener(){

			public void doubleClick(DoubleClickEvent event) {								
				TaxonName taxonname = null;
				try {
					taxonname = (TaxonName) ((StructuredSelection)event.getSelection()).getFirstElement();
				}catch (Exception e){
					e.printStackTrace();
					taxonname = null;
				}
				new OpenNameEditorAction(taxonname).run();
			}
		});		

		nameListExpandItem.setControl(nameListComposite);
		
	}
	
	private void createRecentNamesComposite() {
		recentNamesExpandItem = new ExpandItem(expandBar, SWT.NONE, 1);
		recentNamesExpandItem.setHeight(200);
		recentNamesExpandItem.setExpanded(true);
		recentNamesExpandItem.setText("Recently viewed names");
		recentNamesComposite = new Composite(expandBar, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 5;
		recentNamesComposite.setLayout(layout);
//		recentNamesExpandItem.setHeight(recentNamesComposite.computeSize(
//				SWT.DEFAULT, SWT.DEFAULT).y);
		
		
		rnViewer = new TableViewer(recentNamesComposite, SWT.H_SCROLL
				| SWT.V_SCROLL);
		rnTable = rnViewer.getTable();
		final GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true);
//		gd_table.heightHint = 238;
//		gd_table.widthHint = 443;
		rnTable.setLayoutData(gd_table);
		
		rnViewer.addDoubleClickListener(new IDoubleClickListener(){

			public void doubleClick(DoubleClickEvent event) {								
				TaxonName taxonname = null;
				try {
					taxonname = (TaxonName) ((StructuredSelection)event.getSelection()).getFirstElement();
				}catch (Exception e){
					e.printStackTrace();
					taxonname = null;
				}
				new OpenNameEditorAction(taxonname).run();
			}
		});				
		
		recentNamesExpandItem.setControl(recentNamesComposite);

//		Image image = FileArrangerImages.getImageRegistry().get(
//				IFileArrangerImages.IMG_BOOKMARK);
//		bookmarksExpandItem.setImage(image);
	}

	private void createSearchComposite() {
		searchExpandItem = new ExpandItem(expandBar, SWT.NONE, 2);
		searchExpandItem.setHeight(200);
		searchExpandItem.setExpanded(true);
		searchExpandItem.setText("Search");
		searchComposite = new Composite(expandBar, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		searchComposite.setLayout(gridLayout);

		searchExpandItem.setControl(searchComposite);
		initDataBindings();

//		Image image = FileArrangerImages.getImageRegistry().get(
//				IFileArrangerImages.IMG_FTP);
//		ftpExpandItem.setImage(image);
	}	
	
	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar
	 */
	private void initializeToolBar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu
	 */
	private void initializeMenu() {
		IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();
	}

	@Override
	public void setFocus() {
		// Set the focus
	}
	
	protected DataBindingContext initDataBindings() {
		//
		ObservableListContentProvider viewerContentProviderList = new ObservableListContentProvider();
		viewer.setContentProvider(viewerContentProviderList);
		//
		IObservableMap[] viewerLabelProviderMaps = BeansObservables.observeMaps(viewerContentProviderList.getKnownElements(), TaxonName.class, new String[]{"name"});
		viewer.setLabelProvider(new ObservableMapLabelProvider(viewerLabelProviderMaps));
		//
		IObservableList olObjectListObserveList = BeansObservables.observeList(Realm.getDefault(), Activator.getDefault().getTaxonList(), "objectList");
		viewer.setInput(olObjectListObserveList);
		
		
		ObservableListContentProvider rnViewerContentProviderList = new ObservableListContentProvider();
		rnViewer.setContentProvider(rnViewerContentProviderList);
		//
		IObservableMap[] rnViewerLabelProviderMaps = BeansObservables.observeMaps(viewerContentProviderList.getKnownElements(), TaxonName.class, new String[]{"name"});
		rnViewer.setLabelProvider(new ObservableMapLabelProvider(rnViewerLabelProviderMaps));
		//
		IObservableList rnObjectListObserveList = BeansObservables.observeList(Realm.getDefault(), Activator.getDefault().getRecentNamesList(), "objectList");
		rnViewer.setInput(rnObjectListObserveList);		
		
		//
		DataBindingContext bindingContext = new DataBindingContext();
		//
		//
		return bindingContext;
	}
	
}
