package eu.etaxonomy.taxeditor.prototype2.controller;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;
import eu.etaxonomy.taxeditor.prototype2.view.NameListView;
import eu.etaxonomy.taxeditor.prototype2.view.TaxonomicTreeViewer;

/**
 * Open up a celleditor in the taxonomic tree 
 *  to add a quickname to taxon
 * 
 * @author p.ciardelli
 *
 */
public class ActionAddQuickName extends Action {
	Taxon parentTaxon;
	
	public ActionAddQuickName(Taxon parentTaxon) {
		this.parentTaxon = parentTaxon;
	}
	
	public void run() {
	
		// Create empty Taxon, add it to its parent
		Taxon childTaxon = new Taxon();
		childTaxon.setName(new BotanicalName(Rank.GENUS()));
		
		// TODO getTitleCache is currently returning "null", not null
		//  hence its being set here - change!
		childTaxon.getName().setTitleCache(null);
		parentTaxon.addTaxonomicChild(childTaxon, null, null);
		
		// Add childTaxon to observable tree list, 
		//  causing it to appear in the tree
		Activator.getDefault().getObservableTaxonTreeList().add(childTaxon);
		
		// Open new node
		TaxonomicTreeViewer treeViewer = (TaxonomicTreeViewer) ((NameListView) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().
				findView(NameListView.ID)).getTreeViewer(); 
		treeViewer.revealTaxon(childTaxon);
		
		// Put cursor in empty node
		treeViewer.editElement(childTaxon, 0);
	}
}