package eu.etaxonomy.taxeditor.prototype2.controller;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;

/**
 * For drag and drop - a drag widget with this transfer type can only be dropped onto
 *  a drop widget with the same type.
 *    
 * @author p.ciardelli
 *
 */
public class NameTransfer extends ByteArrayTransfer {

	private static final String TYPENAME = "name_type";
	private static final int NAMETYPEID = registerType (TYPENAME);		
	private static final NameTransfer INSTANCE = new NameTransfer();
	
	/*
	 * The object associated with this transfer event
	 */
	private TaxonNameBase name;
		
	/*
	 * Returns the singleton
	 */
	public static NameTransfer getInstance() {
		return INSTANCE;
	}
	/*
	 * Set transfer data for local use
	 */
	public void setName(TaxonNameBase name) {
		this.name = name;
	}
	/*
	 * Returns the local transfer data
	 */
	public TaxonNameBase getName() {
		return name;
	}
	
	/* 
	 * The type ID is used to identify this transfer
	 */
	@Override
	protected int[] getTypeIds() {
		return new int[] { NAMETYPEID };
	}

	@Override
	protected String[] getTypeNames() {
		return new String[] { TYPENAME } ;
	}

	@Override
	protected void javaToNative(Object object, TransferData transferData) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See nativeToJava below
		byte[] check = TYPENAME.getBytes();
		super.javaToNative(check, transferData);
	}

	@Override
	protected Object nativeToJava(TransferData transferData) {
		Object result = super.nativeToJava(transferData);
		if (isInvalidNativeType(result)) {
			throw new RuntimeException(); //$NON-NLS-1$
		}
		return name;
	}
	
	private boolean isInvalidNativeType(Object result) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See javaToNative above
		return !(result instanceof byte[])
				|| !TYPENAME.equals(new String((byte[]) result));
	}
}
