package eu.etaxonomy.taxeditor.prototype2.view;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.MultiPageEditorPart;

import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * 
 * Generates the tabbed editor with Name view on top and tabs for
 *  "Facts", "Specimen", "Geography", etc.
 * 
 * @author p.ciardelli
 *
 */
public class MultiPageTaxonView extends MultiPageEditorPart {

	public static final String ID = "eu.etaxonomy.taxeditor.prototype2.view.multipagetaxonview";
	private Taxon taxon;
	
	@Override
	protected void createPages() {
		
		try {
			addPage(0, new NameEditorView(), getEditorInput());
			setPageText(0, "Name");

			addPage(1, new NameEditorView(), getEditorInput());
			setPageText(1, "Descriptive");
			
			addPage(2, new NameEditorView(), getEditorInput());
			setPageText(2, "Concepts");
			
			addPage(3, new NameEditorView(), getEditorInput());
			setPageText(3, "Geographic");
			
		} catch (PartInitException e) {			
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		bindPartName();
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void doSaveAs() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean isSaveAsAllowed() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
					"Invalid Input: Must be IFileEditorInput");
		
		// Get taxon from editor input
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			taxon = null;
		}

		// Listen for name changes fired when taxon is saved; 
		//  change tab for this taxon editor accordingly
		taxon.addPropertyChangeListener("name", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				setPartName(taxon.getName().getTitleCache());
			}
		});

		 // Any taxon that has been saved will by necessity have a name;
		 //  only a new taxon will not
		if (taxon.getName().getTitleCache() == null)
			setPartName("New taxon");
		else
			setPartName(taxon.getName().getTitleCache());
		
		setSite(site);
		setInput(input);
	}
}
