package eu.etaxonomy.taxeditor.prototype2.view;

import java.util.Iterator;

import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.AnnotationPainter;
import org.eclipse.jface.text.source.IAnnotationAccess;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.AnnotationMarkerAccess;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.ErrorAnnotation;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.WarningAnnotation;

/**
 * SourceViewer implementation called by NameEditorView.
 *  
 * @author p.ciardelli
 *
 */
public class NameViewer extends SourceViewer {
	
	public NameViewer(Composite parent) {
		super(parent, null, SWT.WRAP | SWT.MULTI | SWT.RESIZE);
				
		// Set name viewer's text to name cache
		this.setDocument(new Document(""), new AnnotationModel());
				
		// Annotations section
		IAnnotationAccess fAnnotationAccess = new AnnotationMarkerAccess();
		
		// To paint the annotations
		AnnotationPainter annotationPainter = new AnnotationPainter(this, fAnnotationAccess);

		// Add ability to paint red squigglies
		annotationPainter.addAnnotationType(ErrorAnnotation.ERROR_TYPE);
		annotationPainter.setAnnotationTypeColor(ErrorAnnotation.ERROR_TYPE, 
				new Color(Display.getDefault(), ErrorAnnotation.ERROR_RGB));

		// Add ability to paint yellow squigglies
		annotationPainter.addAnnotationType(WarningAnnotation.WARNING_TYPE);
		annotationPainter.setAnnotationTypeColor(WarningAnnotation.WARNING_TYPE, 
				new Color(Display.getDefault(), WarningAnnotation.WARNING_RGB));		
		
		this.addPainter(annotationPainter);
	}

	public void setShowError(boolean hasProblem) {
			
		String text = this.getTextWidget().getText();
		
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next(); 
			if (annotation.getType().equals(ErrorAnnotation.ERROR_TYPE))
				this.getAnnotationModel().removeAnnotation(annotation);
		}
		
		// Failed attempt to get multi-line squigglies drawn
		if (hasProblem && text.length() > 0) {
			for (int i = 1; i <= this.getTextWidget().getLineCount(); i++) {
				this.getAnnotationModel().addAnnotation(
						new ErrorAnnotation(i, "Could not parse name."), 
						new Position(0, text.length()));
			}
		}
	}
}