/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Add a new synonym to a taxon.
 * 
 * @author p.ciardelli
 * @created 06.05.2008
 * @version 1.0
 * 
 */
public class AddHeterotypicSynonymToTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddHeterotypicSynonymToTaxonAction.class);

	private static String text = "Create a heterotypic synonym";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.HETEROTYPIC_SYN_ICON);

	Taxon taxon;
	Synonym synonym = null;
	TaxonNameBase synonymName = null;

	private AddHeterotypicSynonymToTaxonAction() {
		super(text);
		setImageDescriptor(image);
	}

	public AddHeterotypicSynonymToTaxonAction(TaxonNameBase synonymName,
			Taxon taxon) {
		this();
		this.synonymName = synonymName;
		this.taxon = taxon;
	}

	public AddHeterotypicSynonymToTaxonAction(Synonym synonym, Taxon taxon) {
		this();
		this.synonymName = synonym.getName();
		this.taxon = taxon;
	}

	public AddHeterotypicSynonymToTaxonAction(String unparsedName, Taxon taxon) {
		this(CdmUtil.parseFullReference(unparsedName, null, null), taxon);
	}

	public void run() {
		if (synonymName != null) {
			SynonymRelationship synonymRelationship = taxon
					.addHeterotypicSynonymName(synonymName);
			synonym = synonymRelationship.getSynonym();
			
			firePropertyChange(ITaxEditorConstants.SYNONYM, null, synonym);
		}
	}

	public Synonym getSynonym() {
		return synonym;
	}
}