/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class AddMisappliedNameToTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddMisappliedNameToTaxonAction.class);

	private static String text = "Add misapplied name to taxon";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);	
	
	private TaxonNameBase name;
	private ReferenceBase citation = null;
	private String microcitation = null;
	private Taxon taxon;

	private Taxon misappliedNameTaxon;

	public AddMisappliedNameToTaxonAction(TaxonNameBase name, ReferenceBase citation,
			String microcitation, Taxon taxon) {
		super();
		this.name = name;
		this.citation = citation;
		this.microcitation = microcitation;
		this.taxon = taxon;
	}

	public AddMisappliedNameToTaxonAction(TaxonNameBase name, Taxon taxon) {
		super();
		this.name = name;
		this.taxon = taxon;
	}

	public void run() {
		Assert.isNotNull(taxon);		
		taxon.addMisappliedName(getMisappliedNameTaxon(), citation, microcitation);
//		TaxEditorPlugin.getDefault().getCdmApp().getTaxonService().saveTaxon(
//				misappliedNameTaxon);
	}
	
	public void setCitation(ReferenceBase citation) {
		this.citation = citation;		
	}
	
	public void setMicrocitation(String microcitation) {
		this.microcitation = microcitation;		
	}	

	public Taxon getMisappliedNameTaxon() {
		if (misappliedNameTaxon == null) {
			misappliedNameTaxon = Taxon.NewInstance(name, null);			
		}
		return misappliedNameTaxon;
	}
}
