/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class ChangeMisappliedNameToSynonymAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeMisappliedNameToSynonymAction.class);
	
	private static String text = "Change synonym to misapplied name";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);

	Action deleteMisappliedNameAction;
	Action addSynonymAction;

	private Taxon misappliedNameTaxon;	
	private Taxon taxon;
	private Synonym synonym;
	
	private ChangeMisappliedNameToSynonymAction() {
		super(text);
		setImageDescriptor(image);
	}
	
	public ChangeMisappliedNameToSynonymAction(Taxon misappliedNameTaxon, Taxon taxon) {
		this();
		this.misappliedNameTaxon = misappliedNameTaxon;
		this.taxon = taxon;
	}
	
	public void run() {
		TaxonNameBase name = misappliedNameTaxon.getName();
		
		deleteMisappliedNameAction = new DeleteMisappliedNameFromTaxonAction(misappliedNameTaxon, taxon);
		deleteMisappliedNameAction.run();
		
		addSynonymAction = new AddHeterotypicSynonymToTaxonAction(name, taxon);
		addSynonymAction.run();
		synonym = ((AddHeterotypicSynonymToTaxonAction) addSynonymAction).getSynonym();
		
		firePropertyChange(ITaxEditorConstants.SYNONYM, null, synonym);		 
	}
}