/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.ui.texteditor.AbstractTextEditor;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;

/**
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class DeleteMisappliedNameFromTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(DeleteMisappliedNameFromTaxonAction.class);

	private Taxon misappliedNameTaxon;
	private Taxon taxon;

	public DeleteMisappliedNameFromTaxonAction(Taxon misappliedNameTaxon,
			Taxon taxon) {
		super();
		this.misappliedNameTaxon = misappliedNameTaxon;
		this.taxon = taxon;
	}

	public void run() {
		for (TaxonRelationship relation : taxon.getTaxonRelations()) {
			if (relation.getType().equals(
					TaxonRelationshipType.MISAPPLIED_NAME_FOR())
					&& relation.getFromTaxon().equals(misappliedNameTaxon)
					&& relation.getToTaxon().equals(taxon)) {
				taxon.removeTaxonRelation(relation);
				firePropertyChange(ITaxEditorConstants.MISAPPLIED_NAME,
						misappliedNameTaxon, null);
				return;
			}
		}
	}
}
