/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

/**
 * @author p.ciardelli
 * @created 14.05.2008
 * @version 1.0
 */
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Delete this taxon from the CDM
 * 
 * @author p.ciardelli
 * 
 */
public class DeleteTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonAction.class);

	private static String text = "Delete taxon from tree";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);

	Taxon taxon;

	public DeleteTaxonAction() {
		super(text);
		setImageDescriptor(image);
	}

	public DeleteTaxonAction(Taxon taxon) {
		this();

		this.taxon = taxon;
	}

	public void run() {

		// If the taxon has children, cancel operation
		// TODO add option to continue, and delete children
		if (taxon.hasTaxonomicChildren()) {
			MessageDialog.openInformation(UiUtil.getShell(), "Cannot delete taxon", 
					"'" + CdmUtil.getDisplayName(taxon) + "' has taxonomic children. " +
					"These must be manually deleted before their parent.");
			return;
		}
		
		TaxEditorPlugin.getDefault().removeSessionTaxon(taxon);

		// Close taxon's editor, if any is active
		try {
			UiUtil.closeEditor(taxon, true);
		} catch (PartInitException e) {
			e.printStackTrace();
		}

		// Remove from recent names list
		TaxEditorPlugin.getDefault().getObservableRecentNamesList().remove(
				taxon);		
		
		Taxon parentTaxon = taxon.getTaxonomicParent();

		if (parentTaxon != null) {
			for (TaxonRelationship taxonRelation : parentTaxon
					.getTaxonRelations()) {

				TaxonRelationshipType type = taxonRelation.getType();
				Taxon fromTaxon = taxonRelation.getFromTaxon();

				// Beware of phantom taxon relations with a "NULL" fromTaxon
				if (fromTaxon == null) {
					continue;
				}

				if (type.equals(TaxonRelationshipType
						.TAXONOMICALLY_INCLUDED_IN())
						&& fromTaxon.equals(taxon)) {
					parentTaxon.removeTaxonRelation(taxonRelation);
					TaxEditorPlugin.getDefault().getCdmApp().getTaxonService()
							.saveTaxon(parentTaxon);
					break;
				}
			}
		}
		
		// Check whether taxon has already been saved to the CDM
		UUID taxonUuid = taxon.getUuid();
		if (TaxEditorPlugin.getDefault().getCdmApp().getTaxonService()
				.getTaxonByUuid(taxonUuid) != null) {
			
			// Remove the taxon from the CDM
			TaxEditorPlugin.getDefault().getCdmApp().getTaxonService()
					.removeTaxon(taxon);
		}

	}
}