/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;
import eu.etaxonomy.taxeditor.propertysheet.PropertySourceAdapter;

/**
 * Open up a celleditor in the taxonomic tree 
 *  to add a quickname to taxon
 * 
 * @author p.ciardelli
 * @created 29.05.2008
 * @version 1.0
 */
public class AddQuickNameAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddQuickNameAction.class);
	
	private static String text = "Add child taxon with quick name";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.QUICK_ADD_ICON);	
	
	private Taxon parentTaxon;
	private Taxon quickNameTaxon;
	
	private AddQuickNameAction() {
		super(text);
		setImageDescriptor(image);	
	}
	
	public AddQuickNameAction(Taxon parentTaxon) {
		this();
		
		this.parentTaxon = parentTaxon;
	}
	
	public void run() {
		
		// Create empty Taxon, add it to its parent
		if (UiUtil.getNameCodePreference().equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			quickNameTaxon = Taxon.NewInstance(ZoologicalName.NewInstance(Rank.GENUS()), parentTaxon.getSec());
		} else {
			quickNameTaxon = Taxon.NewInstance(BotanicalName.NewInstance(Rank.GENUS()), parentTaxon.getSec());
		}
		parentTaxon.addTaxonomicChild(quickNameTaxon, null, null);
		
		TaxEditorPlugin.getDefault().addSessionTaxon(quickNameTaxon);
		
		// ULTRA HACKY, ULTRA MYSTERIOUS - if this is not called, name created with quick name
		// then deleted in the same session causes error 
		// "org.springframework.dao.InvalidDataAccessApiUsageException: deleted object would be re-saved by cascade" 
		(new PropertySourceAdapter(quickNameTaxon.getName())).getPropertySource();
		
		firePropertyChange(ITaxEditorConstants.QUICK_NAME_TAXON, null, quickNameTaxon);
		
		TaxonomicTreeViewer treeViewer = UiUtil.getTreeViewer();
		if (treeViewer == null) {
			return;
		}
		
		// Open new node
		treeViewer.revealTaxon(quickNameTaxon);
		
		// Put cursor in empty node
		treeViewer.editElement(quickNameTaxon, 0);
	}

	public Taxon getQuickNameTaxon() {
		return quickNameTaxon;
	}
}