/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.ChangeMisappliedNameToSynonymAction;
import eu.etaxonomy.taxeditor.actions.cdm.ChangeSynonymToMisappliedNameAction;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class ChangeCompositeToMisappliedNameAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeCompositeToMisappliedNameAction.class);

	private static String text = "Change synonym to misapplied name";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);

	private Synonym synonym;
	private Taxon taxon;
	private Taxon misappliedNameTaxon;

	private Action misappliedNameCdmAction;
	private Action misappliedNameCdmUndoAction;

	private IUndoableOperation operation;
	private NameComposite composite;
	private IManagedForm managedForm;

	private ChangeCompositeToMisappliedNameAction() {
		super(text);
		setImageDescriptor(image);
	}

	public ChangeCompositeToMisappliedNameAction(NameComposite composite,
			IManagedForm managedForm) {
		this();

		if (composite.getData() instanceof Synonym) {
			this.synonym = (Synonym) composite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a composite with a Synonym in its data field.");
		}

		Composite parent = composite.getParent();
		if (parent.getParent() == null
				|| !(parent.getParent().getData() instanceof Taxon)) {
			throw new IllegalArgumentException(
					"The parent of the DescriptionElementComposite's parent does not have a Taxon in its data field as required.");
		}

		this.taxon = (Taxon) parent.getParent().getData();
		this.composite = composite;
		this.managedForm = managedForm;

		operation = new MisappliedNameOperation();

	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class MisappliedNameOperation extends AbstractOperation {

		public MisappliedNameOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			misappliedNameCdmAction = new ChangeSynonymToMisappliedNameAction(
					synonym, taxon);
			misappliedNameCdmAction
					.addPropertyChangeListener(new IPropertyChangeListener() {
						public void propertyChange(
								org.eclipse.jface.util.PropertyChangeEvent event) {
							if (event.getProperty().equals(
									ITaxEditorConstants.MISAPPLIED_NAME)) {

								misappliedNameTaxon = (Taxon) event.getNewValue();
								composite.setData(misappliedNameTaxon);
								composite.transform(NameComposite.MISAPPLIED_NAME);

								EditorGroupComposite groupComposite = UiUtil
										.getMisappliedNameGroupComposite(managedForm);
								if (groupComposite == null) {
									groupComposite = UiUtil
											.createMisappliedNameGroupComposite(managedForm);
								}
//								composite.setParent(groupComposite);
							}
						}
					});
			misappliedNameCdmAction.run();
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return execute(monitor, info);
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			logger.warn("performing undo");
			misappliedNameCdmUndoAction = new ChangeMisappliedNameToSynonymAction(
					misappliedNameTaxon, taxon);
			misappliedNameCdmUndoAction
					.addPropertyChangeListener(new IPropertyChangeListener() {

						public void propertyChange(PropertyChangeEvent event) {
							if (event.getProperty().equals(
									ITaxEditorConstants.SYNONYM)) {
								synonym = (Synonym) event.getNewValue();
								composite.setData(misappliedNameTaxon);
								composite.transform(NameComposite.HETEROTYPIC_SYNONYM);
							}
						}
					});
			misappliedNameCdmUndoAction.run();
			return Status.OK_STATUS;
		}
	}
}