/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;

/**
 * Opens a name editor for a new taxon with a parent taxon
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class OpenNewChildNameEditorAction extends Action {
	private static final Logger logger = Logger
			.getLogger(OpenNewChildNameEditorAction.class);

	private static String text = "Add child taxon";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.ADD_CHILD_TAXON_ICON);

	private Taxon taxon;
	private Taxon parentTaxon;
	public static final String ID = "eu.etaxonomy.taxeditor.actions.opennameeditoraction"; //$NON-NLS-1$
	
	private OpenNewChildNameEditorAction() {
		super(text);
		setImageDescriptor(image);
		setId(ID);
	}
	
	public OpenNewChildNameEditorAction(Taxon parentTaxon) {
		this();
		
		this.parentTaxon = parentTaxon;
	}
	
	public void run() {
		
		if (UiUtil.getNameCodePreference().equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			taxon = Taxon.NewInstance(ZoologicalName.NewInstance(Rank.GENUS()), parentTaxon.getSec());
		} else {
			taxon = Taxon.NewInstance(BotanicalName.NewInstance(Rank.GENUS()), parentTaxon.getSec());
		}
		parentTaxon.addTaxonomicChild(taxon, null, null);
		
		new OpenTaxonEditorAction(taxon).run();
	}
}