/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.model.NameEditorInput;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * Opens a taxon editor for an existing or new taxon.
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class OpenTaxonEditorAction extends Action {
	private static final Logger logger = Logger
			.getLogger(OpenTaxonEditorAction.class);

	private static String editTaxonText = "Edit taxon";
	private static String newTaxonText = "Add new taxon";
//	private ImageDescriptor image = TaxEditorPlugin.getDefault()
//			.getImageDescriptor(ITaxEditorConstants.OPEN_TAXON_ICON);
	private ImageDescriptor image = null;
	
	private Taxon taxon;
	public static final String ID = "eu.etaxonomy.taxeditor.actions.opennameeditoraction"; //$NON-NLS-1$
	
	public OpenTaxonEditorAction(Taxon taxon) {
		this();
		
		this.taxon = taxon;
		if (taxon != null) {
			setText(editTaxonText);
		}
	}

	public OpenTaxonEditorAction() {
		super(newTaxonText);
		setImageDescriptor(image);	
		setId(ID);
	}
	
	@Override
	public void run() {
		
		// Initialize taxon
		if (taxon == null) {
			
			// Passing the parser an empty string will return a new
			//  new name object with the name class specified in the
			//  preference store, i.e. BotanicalName
			TaxonNameBase name = CdmUtil.parseFullReference("", null, null);
			name.setFullTitleCache("", false);
			this.taxon = Taxon.NewInstance(name, CdmUtil.getSessionDefaultSec());
		} else {
			
			// If this taxon is not visible in the tree, open node
			TaxonomicTreeViewer treeViewer = UiUtil.getTreeViewer(); 
			if (treeViewer != null) {
				treeViewer.revealTaxon(taxon);
			}
			
			// Add to recent names list if not already there		
			if (!TaxEditorPlugin.getDefault().getObservableRecentNamesList().contains(taxon)) {
				TaxEditorPlugin.getDefault().getObservableRecentNamesList().add(0, taxon);
			}
		}
		
		IEditorInput input = new NameEditorInput(taxon);
    	try {
			UiUtil.openEditor(input, MultiPageTaxonEditor.ID);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
		// For clients who re-use the same instance of this class, 
		//	taxon is re-initalized to ensure a new Taxon is opened every time
		taxon = null;
	}
}