package eu.etaxonomy.taxeditor.datasource;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

public class CdmDataSourceDialog extends Dialog {
	private List list;
	private List list_1;
	private Text text_6;
	private Text text_5;
	private Text text_4;
	private Text text_2;
	private Text text_3;
	private Text text;
	private static final Logger logger = Logger
			.getLogger(CdmDataSourceDialog.class);
	private CCombo datasourceTypesCombo;
	private Composite composite;
	private Composite container;

	/**
	 * Create the dialog
	 * @param parentShell
	 */
	public CdmDataSourceDialog(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * Create contents of the dialog
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		container = (Composite) super.createDialogArea(parent);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		container.setLayout(gridLayout);

		final ListViewer listViewer = new ListViewer(container, SWT.BORDER);
		list = listViewer.getList();
		final GridData gd_list = new GridData(SWT.LEFT, SWT.FILL, false, true);
		gd_list.widthHint = 162;
		list.setLayoutData(gd_list);
		
		listViewer.setContentProvider(new ArrayContentProvider());
		listViewer.setLabelProvider(new LabelProvider() {
			/* (non-Javadoc)
			 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
			 */
			public String getText(Object element) {
				if (element instanceof ICdmDataSource) {
					return ((ICdmDataSource) element).getName();
				} 
				return super.getText(element);
			}
		});
//		list.addListener(SWT. .PaintItem, new Listener() {
//			@Override
//			public void handleEvent(Event event) {
//				Object data = event.item.getData();
//				if (!(data instanceof ICdmDataSource)) {
//					Font italics = TaxEditorPlugin.getDefault().getFont
//							(ITaxEditorConstants.MENU_ITEM_ITALICS_FONT);
//					((Control) event.item).setFont(italics);
//				}
//			}
//		});
		
		java.util.List<ICdmDataSource> dataSources = 
			TaxEditorPlugin.getDefault().getDataSources();
		
		listViewer.setInput(dataSources);
		listViewer.add("<< Add data source >>");
		
		listViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				Object selection = 
						((IStructuredSelection) event.getSelection()).
						getFirstElement();
				if (selection instanceof ICdmDataSource) {
					createExistingDatasource((ICdmDataSource) selection);
				} else {
//					newDataSource(CdmDataSource.);
				}
			}
		});


		//
		return container;
	}

	private void populateComboBoxItems() {
		// TODO: replace w cdmApp.getDatabaseService().getDatabaseEnum();
		for (DatabaseTypeEnum type : DatabaseTypeEnum.getAllTypes()) {
			datasourceTypesCombo.add(type.getName());
		}
	}

	/**
	 * Create contents of the button bar
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 375);
	}
	
	private void createExistingDatasource(ICdmDataSource dataSource) {
		composite = new Composite(container, SWT.NONE);
		final GridData gd_composite = new GridData(SWT.FILL, SWT.TOP, true, true);
		composite.setLayoutData(gd_composite);
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		composite.setLayout(gridLayout_1);
		
		final Label datasourceNameLabel = new Label(composite, SWT.NONE);
		datasourceNameLabel.setText("Datasource Name:");

		text = new Text(composite, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		final Label databaseTypeLabel = new Label(composite, SWT.NONE);
		databaseTypeLabel.setText("Database Type:");

		datasourceTypesCombo = new CCombo(composite, SWT.BORDER);
		final GridData gd_list_1 = new GridData(SWT.FILL, SWT.CENTER, false, false);
		datasourceTypesCombo.setLayoutData(gd_list_1);
		populateComboBoxItems();
		
//		BeanDefinition bean = dataSource.getDatasourceBean();
		
		// Redraw dialog container
		container.layout();
	}
	
	private void createMySqlForm(ICdmDataSource dataSource) {
		final Group authenticationGroup = new Group(composite, SWT.NONE);
		authenticationGroup.setText("Authentication");
		authenticationGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
		final GridLayout gridLayout_2 = new GridLayout();
		gridLayout_2.numColumns = 2;
		authenticationGroup.setLayout(gridLayout_2);

		final Label loginNameLabel = new Label(authenticationGroup, SWT.NONE);
		loginNameLabel.setText("Login Name:");

		text_3 = new Text(authenticationGroup, SWT.BORDER);
		text_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		final Label passwordLabel = new Label(authenticationGroup, SWT.NONE);
		passwordLabel.setText("Password:");

		text_2 = new Text(authenticationGroup, SWT.BORDER);
		text_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		final Label savePasswordLabel = new Label(authenticationGroup, SWT.NONE);
		savePasswordLabel.setText("Save password:");

		final Button button = new Button(authenticationGroup, SWT.CHECK);

		final Group locationGroup = new Group(composite, SWT.NONE);
		locationGroup.setText("Location");
		final GridData gd_locationGroup = new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1);
		locationGroup.setLayoutData(gd_locationGroup);
		final GridLayout gridLayout_3 = new GridLayout();
		gridLayout_3.numColumns = 2;
		locationGroup.setLayout(gridLayout_3);

		final Label hostLabel = new Label(locationGroup, SWT.NONE);
		hostLabel.setText("Host:");

		text_4 = new Text(locationGroup, SWT.BORDER);
		text_4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		final Label portLabel = new Label(locationGroup, SWT.NONE);
		portLabel.setText("Port <3306>:");

		text_5 = new Text(locationGroup, SWT.BORDER);
		text_5.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		final Label databaseLabel = new Label(locationGroup, SWT.NONE);
		databaseLabel.setText("Database:");

		text_6 = new Text(locationGroup, SWT.BORDER);
		text_6.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		new Label(composite, SWT.NONE);

		final Button saveButton = new Button(composite, SWT.NONE);
		final GridData gd_saveButton = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		gd_saveButton.widthHint = 76;
		saveButton.setLayoutData(gd_saveButton);
		saveButton.setText("Save");
	}
}
