/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;

/**
 * Creates a menu that is opened by right-clicking on a <code>Control</code>.
 * <p>
 * Note that menu items are added anew each time the menu is opened.
 * </p>
 * 
 * @author p.ciardelli
 * @created 26.05.2008
 * @version 1.0
 */
public class ContextMenu {
	private static final Logger logger = Logger.getLogger(ContextMenu.class);
	
	private Control control;
	private Menu menu;
	private MenuManager menuManager;
	private List actions;
	
	/**
	 * Constructor for top-level menu items.
	 * 
	 * @param control
	 */
	public ContextMenu(Control control) {
		this.control = control;
		
		createContextMenu();
	}
	
	private void createContextMenu() {
		menuManager = new MenuManager();
		menu = menuManager.createContextMenu(control);
		control.setMenu(menu);
		
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				for (Object action : getActions()) {
					if (action instanceof Action) {
						manager.add((Action) action);
					}
					if (action instanceof Separator) {
						manager.add((Separator) action);
					}
					if (action instanceof MenuManager) {
						manager.add((MenuManager) action);
					}
				}
			}
		});
	}

	private List getActions() {
		if (actions == null) {
			actions = new ArrayList();
		}
		return actions;
	}
	
	public void addAction(Action action) {
		getActions().add(action);
	}

	public void removeAction(Action action) {
		getActions().remove(action);
	}
	
	public void addSeparator() {
		getActions().add(new Separator());
	}

	public void addSubmenu(MenuManager submenu) {
		getActions().add(submenu);
	}
}