/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 27.06.2008
 * @version 1.0
 */
public class TaxonomicTreeContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeContentProvider.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		
		if (parentElement instanceof Taxon) {
			return getTaxonChildren((Taxon) parentElement).toArray();
		}
		
		if (!(parentElement instanceof Collection)) {
			throw new IllegalArgumentException(
					"getChildren() expects either a Collection or a Taxon object."); //$NON-NLS-1$	
		}
		
		List<Taxon> children = new ArrayList<Taxon>();
		for (Object parent : (Collection) parentElement) {
			if (!(parent instanceof Taxon)) {
				throw new IllegalArgumentException(
						"parentElement Collection can only contain Taxon objects."); //$NON-NLS-1$				
			}
			
			children.addAll(getTaxonChildren((Taxon) parent));
		}
		return children.toArray();
	}

	public Set<Taxon> getTaxonChildren(Taxon parentTaxon) {

		return TaxEditorPlugin.getDefault().getSessionTaxonomicChildren(
				parentTaxon);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		if (!(element instanceof Taxon)) {
			throw new IllegalArgumentException(
					"getParent() expects a Taxon object."); //$NON-NLS-1$				
		}
		return ((Taxon) element).getTaxonomicParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		if (!(element instanceof Taxon)) {
			throw new IllegalArgumentException(
					"getParent() expects a Taxon object."); //$NON-NLS-1$				
		}
		
		return getChildren(element).length > 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		
		// BIG QUESTION: should getElements return only root taxa, or all taxa?
		// CONFLICTING INFO ONLINE!!!
//		return TaxEditorPlugin.getDefault().getObservableSessionTaxa().toArray();
		
		return TaxEditorPlugin.getDefault().getSessionRootTaxa().toArray();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
	 *      java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub
	}
}