package eu.etaxonomy.taxeditor.propertysheet;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.Annotation;

public class AnnotationsDialog extends Dialog {

	private Text text;
	private Table table;
	private static final Logger logger = Logger
			.getLogger(AnnotationsDialog.class);
	protected Object result;
	protected Shell shell;
	
	private WritableList list = new WritableList();
	private Annotation selectedAnnotation;

	/**
	 * Create the dialog
	 * @param parent
	 * @param style
	 */
	public AnnotationsDialog(Shell parent, int style) {
		super(parent, style);
	}

	/**
	 * Create the dialog
	 * @param parent
	 */
	public AnnotationsDialog(Shell parent) {
		this(parent, SWT.NONE);
	}

	public AnnotationsDialog(Shell parent, Set<Annotation> annotations) {
		this(parent, SWT.NONE);
		for (Annotation annotation : annotations) {
			list.add(annotation);
		}
	}

	/**
	 * Open the dialog
	 * @return the result
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return result;
	}

	/**
	 * Create contents of the dialog
	 */
	protected void createContents() {
		
		// Create shell for popup dialog
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setLayout(new FillLayout());
		shell.setSize(500, 375);
		shell.setText("Annotations");
		
		// Create composite for entire shell
		final Composite composite = new Composite(shell, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		composite.setLayout(gridLayout);

		text = new Text(composite, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

		final Button addButton = new Button(composite, SWT.NONE);
		addButton.setLayoutData(new GridData());
		addButton.setText("Add");
		addButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDown(MouseEvent e) {
				Annotation annotation = Annotation.NewInstance(text.getText(), null);
				list.add(annotation);
				text.setText("");
			}
		});

		final TableViewer tableViewer = new TableViewer(composite, SWT.BORDER | SWT.MULTI);
		table = tableViewer.getTable();
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
		table.setLinesVisible(false);
		table.setHeaderVisible(false);
//		table.s
		new Label(composite, SWT.NONE);

		ObservableListContentProvider providerList = new ObservableListContentProvider();
		tableViewer.setContentProvider(providerList);
		
		IObservableMap[] providerMaps = BeansObservables.observeMaps(
				providerList.getKnownElements(), Annotation.class, new String[]{"text"});
		tableViewer.setLabelProvider(new ObservableMapLabelProvider(providerMaps));
		tableViewer.setInput(list);
		
		final MenuManager manager = new MenuManager();
		final Menu menu = manager.createContextMenu(table);
		table.setMenu(menu);
		
		table.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				
				Object data = e.item.getData();
				
				if (data instanceof Annotation) {
					manager.removeAll();
					manager.add(new RemoveAnnotationAction((Annotation) data));
				}
			}
		});

		final Button cancelButton = new Button(composite, SWT.NONE);
		cancelButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		cancelButton.setText("Cancel");
		cancelButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				shell.dispose();
			}
		});
			
		final Button okButton = new Button(composite, SWT.NONE);
		final GridData gd_okButton = new GridData();
		okButton.setLayoutData(gd_okButton);
		okButton.setText("OK");
		okButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				result = new HashSet(list);
				shell.dispose();
			}
		});
	}
	
	public class RemoveAnnotationAction extends Action {
		
		private Annotation annotation;

		RemoveAnnotationAction(Annotation annotation) {
			setText("Remove");
			this.annotation = annotation;
		}
		
		public void run() {
			list.remove(annotation);
		}
	}
}