/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.propertysheet.AnnotationPropertySource;
import eu.etaxonomy.taxeditor.propertysheet.AnnotationsPropertyDescriptor;
import eu.etaxonomy.taxeditor.propertysheet.MarkersPropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.IReferenceSearch;
import eu.etaxonomy.taxeditor.propertysheet.reference.NomenclaturalReferencePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferenceSearchDescriptor;

/**
 * @author p.ciardelli
 * @created 06.05.2008
 * @version 1.0
 */
public class NonViralNamePropertySource implements IPropertySource {
	private static final Logger logger = Logger
			.getLogger(NonViralNamePropertySource.class);

	// The name whose properties are being displayed
	NonViralName name;
	
	// If this is a property with a parent, the parent's property ID
	private String parentid = "";
	
	PropertyChangeSupport propertyChangeSupport;
	
    // Property unique keys
	public static final String P_ID_PARSED = "parsed";
	public static final String P_ID_RANK = "rank";
	public static final String P_ID_TITLECACHE = "titlecache";
	public static final String P_ID_UNINOMIAL = "uninomial";
	public static final String P_ID_INFRAGENERICEP = "infragenericep";
	public static final String P_ID_SPECIESEP = "speciesep";
	public static final String P_ID_INFRASPECIFICEP = "infraspecificep";
	public static final String P_ID_APPENDEDPHRASE = "appendedphrase";
	public static final String P_ID_HOMOTYPICALGROUP = "homogroup";
	public static final String P_ID_AUTHORSHIPCACHE = "authorshipcache";
	public static final String P_ID_NAMERELATIONS = "namerelations";
	public static final String P_ID_TYPE = "type";
	public static final String P_ID_ANNOTATIONS = "annotations";
	public static final String P_ID_NOMSTATUS = "nomstatus";
	public static final String P_ID_UUID = "uuid";	
	public static final String P_ID_NUM_OF_BASES = "numberofbases";
	public static final String P_ID_MARKERS = "markers";
	public static final String P_ID_CREATED = "created";
	public static final String P_ID_CREATEDBY = "createdby";
	public static final String P_ID_NOMENCLATURAL_CODE = "nomenclaturalcode";
	public static final String P_ID_NOMENCLATURAL_REF = "nomenclaturalref";
	public static final String P_ID_NOMENCLATURAL_MICROREF = "nomenclaturalmicroref";
	
    // Property display keys
	// Note: for an explanation of the sorting prefixes ("04:"), 
	//  @see eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry
	public static final String P_PARSED = "00:Parsing Status";
	public static final String P_RANK = "02:Rank";
	public static final String P_TITLECACHE = "03:Name Cache";
	public static final String P_UNINOMIAL = "04:Uninomial";
	public static final String P_INFRAGENERICEP = "05:Infrageneric Epithet";
	public static final String P_SPECIESEP = "06:Specific Epithet";
	public static final String P_INFRASPECIFICEP = "07:Infraspecific Epithet";
	public static final String P_APPENDEDPHRASE = "09:Appended Phrase";
	public static final String P_HOMOTYPICALGROUP = "09:Homotypical Group";
	public static final String P_AUTHORSHIPCACHE = "08:Authorship";
	public static final String P_NAMERELATIONS = "11:Name Relations";
	public static final String P_TYPE = "12:Type";
	public static final String P_ANNOTATIONS = "14:Annotations";
	public static final String P_NOMSTATUS = "10:Nomenclatural Status";
	public static final String P_UUID = "01:UUID";
	public static final String P_NUM_OF_BASES = "12:Number of Bases";
	public static final String P_MARKERS = "13:Markers";
	public static final String P_CREATED = "15:Created";
	public static final String P_CREATEDBY = "16:Created By";
	public static final String P_NOMENCLATURAL_CODE = "02:Nomenclatural Code";
	public static final String P_NOMENCLATURAL_REF = "09:Nomenclatural Reference";
	public static final String P_NOMENCLATURAL_MICROREF = "10:Micro Reference";
	
	protected static final String[] TOP_LEVEL_PROPERTIES = new String[] {
			P_ID_PARSED, P_ID_RANK, P_ID_TITLECACHE, P_ID_ANNOTATIONS, P_ID_NOMSTATUS, 
			P_ID_NAMERELATIONS, P_ID_UUID, P_ID_NOMENCLATURAL_REF, P_ID_NOMENCLATURAL_MICROREF,
			P_ID_TYPE, P_ID_CREATED, P_ID_CREATEDBY, P_ID_NOMENCLATURAL_CODE };	
	
	// ***********************************************************
	//		RANKS
	// ***********************************************************
	Rank[] ranks = null;
	String[] P_RANK_MENU = null;
	private void populateRanks() {
		
		// Get terms from rank vocabulary
		List<Rank> ranksList = new ArrayList<Rank>();
		ranksList.addAll(UiUtil.getPreferredRanks());
		
		// Populate ranks menu labels 
		List<String> ranksMenuList = new ArrayList<String>();
		for (Rank rank : ranksList) {
			ranksMenuList.add(rank.getLabel());
		}
		
		// Add an empty element for "nothing selected" in the rank drop-down 
		ranksList.add(0, null);
		ranksMenuList.add(0, "");
		
		// Convert rank lists to array
		ranks = ranksList.toArray(new Rank[ranksList.size()]);
		P_RANK_MENU = ranksMenuList.toArray(new String[ranksMenuList.size()]);
	}
	
	static String[] P_NOMENCLATURALCODE_MENU = new String[] { "ICZN", "ICBN" };
	static String[] P_NOMREF_TYPE_MENU = new String[] { "Article", "Bibtex", "Book", "Book Section", "Generic" };	

	// ***********************************************************
	//		NOMENCLATURAL STATUS
	// ***********************************************************
	static TermVocabulary<NomenclaturalStatusType> nomStatusVocabulary = 
			TaxEditorPlugin.getDefault().getNomStatusVocabulary();
	static NomenclaturalStatusType[] nomStatusTypes = null;
	static String[] P_NOMSTATUS_MENU = null;
	private static void populateNomStatusTypes() {
		
		// Get terms from nom status vocabulary
		List<NomenclaturalStatusType> nomStatusTypesList = new ArrayList<NomenclaturalStatusType>();
		nomStatusTypesList.addAll(nomStatusVocabulary.getTerms());
		
		// Populate nom status type menu labels 
		List<String> nomStatusTypesMenuList = new ArrayList<String>();
		for (NomenclaturalStatusType nomStatusType : nomStatusTypesList) {
			nomStatusTypesMenuList.add(nomStatusType.getLabel());
		}
		
		// Add an empty element for "nothing selected" in the rank drop-down 
		nomStatusTypesList.add(0, null);
		nomStatusTypesMenuList.add(0, "");
		
		// Convert rank lists to array
		nomStatusTypes = nomStatusTypesList.toArray(new NomenclaturalStatusType[nomStatusTypesList.size()]);
		P_NOMSTATUS_MENU = nomStatusTypesMenuList.toArray(new String[nomStatusTypesMenuList.size()]);
	}	
		
	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	/**
	 * Add descriptor for a given property.
	 * <p>
	 * Notes on Descriptor:
	 * </p>
	 * <ul>
	 *  <li>PropertyDescriptor - uneditable cell value</li>
	 *  <li>ComboBoxPropertyDescriptor - dropdown list, property supplied must be integer-based</li>
	 *  <li>TextPropertyDescriptor - editable cell value</li>
	 * </ul>
	 *  <p>
	 *  If any descriptor calls setCategory, all descriptors w/out a category are put in 
	 *  category "Misc".
	 *  <p>
	 *  <code>descriptor.setFilterFlags (new String[] { IPropertySheetEntry.FILTER_ID_EXPERT })</code> -
	 *   this descriptor shown under advanced properties
	 *  
	 * @param id
	 */
	protected void addDescriptor(String id) {
		
		// Parsed: reports whether parsing was successful
		if (id.equals(P_ID_PARSED)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_PARSED, P_PARSED));
		}
		
		// Rank menu
		if (id.equals(P_ID_RANK)) {
			if (ranks == null) {
				populateRanks();
			}
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_RANK, P_RANK, P_RANK_MENU));
		}
		
		// Title cache
		if (id.equals(P_ID_TITLECACHE)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_TITLECACHE, P_TITLECACHE));
		}
		
		// Uninomial (aka Genus)
		if (id.equals(P_ID_UNINOMIAL)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_UNINOMIAL, P_UNINOMIAL));
		}
		
		// Infrageneric epithet
		if (id.equals(P_ID_INFRAGENERICEP)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_INFRAGENERICEP, P_INFRAGENERICEP));
		}
		
		// Specific epithet
		if (id.equals(P_ID_SPECIESEP)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_SPECIESEP, P_SPECIESEP));
		}
		
		// Infraspecific epithet
		if (id.equals(P_ID_INFRASPECIFICEP)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_INFRASPECIFICEP, P_INFRASPECIFICEP));
		}
		
		// Appended phrase
		if (id.equals(P_ID_APPENDEDPHRASE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_APPENDEDPHRASE, P_APPENDEDPHRASE));
		}
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE));
		}
		
		// Homotypical group
		if (id.equals(P_ID_HOMOTYPICALGROUP)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_HOMOTYPICALGROUP, P_HOMOTYPICALGROUP));
		}
		
		// Name relations, listed in custom property descriptor
		if (id.equals(P_ID_NAMERELATIONS)) {
			descriptors.addElement(
					new NameRelationsPropertyDescriptor(P_ID_NAMERELATIONS, P_NAMERELATIONS, name) {
						protected void saveNameRelations(Set set) {
							setPropertyValue(P_ID_NAMERELATIONS, set);
						}
					}
			);
		};
		
		// Type
		if (id.equals(P_ID_TYPE)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_TYPE, P_TYPE));
		}
		
		// Annotations, listed in custom property descriptor
		if (id.equals(P_ID_ANNOTATIONS)) {
			descriptors.addElement(
					new AnnotationsPropertyDescriptor(P_ID_ANNOTATIONS, P_ANNOTATIONS, name) {
						protected void saveAnnotations(Set set) {
							
							setPropertyValue(P_ID_ANNOTATIONS, set);
						}
					}
			);
		};
		
		// Nomenclatural status
		if (id.equals(P_ID_NOMSTATUS)) {
			if (nomStatusTypes == null) {
				populateNomStatusTypes();
			}
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_NOMSTATUS, P_NOMSTATUS, P_NOMSTATUS_MENU));
		}		
		
		// UUID
		if (id.equals(P_ID_UUID)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_UUID, P_UUID));
		}
		
		// Markers
		if (id.equals(P_ID_MARKERS)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_MARKERS, P_MARKERS));
		}
		
		// Taxon bases referring to this name
		if (id.equals(P_ID_NUM_OF_BASES)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_NUM_OF_BASES, P_NUM_OF_BASES));
		}
		
		// Object created when
		if (id.equals(P_ID_CREATED)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_CREATED, P_CREATED));
		}
		
		// Object created by
		if (id.equals(P_ID_CREATEDBY)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_CREATEDBY, P_CREATEDBY));
		}
		
		// Nomenclatural code used by parser
		if (id.equals(P_ID_NOMENCLATURAL_CODE)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_NOMENCLATURAL_CODE, P_NOMENCLATURAL_CODE));
		}
		
		// Nomenclatural reference
		if (id.equals(P_ID_NOMENCLATURAL_REF)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_NOMENCLATURAL_REF,P_NOMENCLATURAL_REF));			
//			descriptors.addElement(
//					new ReferenceSearchDescriptor(P_ID_NOMENCLATURAL_REF, P_NOMENCLATURAL_REF, IReferenceSearch.NOMREF) {
//						protected void saveReference(ReferenceBase reference) {
//							if (reference instanceof INomenclaturalReference) {
//								setPropertyValue(P_ID_NOMENCLATURAL_REF, reference);
//							} else {
//								MessageDialog.openError(UiUtil.getShell(), "Reference type error", "The selected reference cannot be used as a nomenclatural reference.");
//							}
//						}
//			});
		}
		
		// Microreference
		if (id.equals(P_ID_NOMENCLATURAL_MICROREF)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_NOMENCLATURAL_MICROREF,P_NOMENCLATURAL_MICROREF));
		}	
	}
	
	/**
	 * Constructor for top level property fields. All fields that are not subfields
	 *  should be listed here.
	 * @param name
	 */
	public NonViralNamePropertySource(NonViralName name) {
		this(name, null, TOP_LEVEL_PROPERTIES);
	}

	public NonViralNamePropertySource(NonViralName name, 
			String parentid, String[] keys) {
		this.name = name;
		this.parentid = parentid;
		for (String key : keys) {
			addDescriptor(key);
		}
	}

	public Object getEditableValue() {
		return this;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	public Object getPropertyValue(Object id) {
		
		// Parsed: reports whether parsing was successful		
        if (id.equals(P_ID_PARSED)) {
			return name.getHasProblem() == true ? "problem" : "parsed" ;
        }
        
		// Rank menu        
		if (id.equals(P_ID_RANK)) {
        	if (this.name.getRank() == null) {
        		return 0;
        	}
        	Rank rank = this.name.getRank();
        	for (int i = 0; i < ranks.length; i++) {
        		if (ranks[i] == null) continue;
        		if (rank.getUuid().equals(ranks[i].getUuid())) 
        			return i;
        	}
        	return 0;
        }
		
		// Title cache		
        if (id.equals(P_ID_TITLECACHE)) {
        	
        	// If the name has not been parsed, only show the title cache
//        	if (name.getHasProblem()) {
        	if (name.getRank() == null) {
        		return CdmUtils.Nz(name.getTitleCache());
        	} else {
        		return new NonViralNamePropertySource(name, P_ID_TITLECACHE, getNameFields());
        	}
        }
        
		// Uninomial (aka Genus)        
		if (id.equals(P_ID_UNINOMIAL)) {
			return CdmUtils.Nz(name.getGenusOrUninomial());
		}
		
		// Infrageneric epithet		
        if (id.equals(P_ID_INFRAGENERICEP)) {
			return CdmUtils.Nz(name.getInfraGenericEpithet());
        }
        
		// Specific epithet        
        if (id.equals(P_ID_SPECIESEP)) {
			return CdmUtils.Nz(name.getSpecificEpithet());
        }
        
		// Infraspecific epithet        
        if (id.equals(P_ID_INFRASPECIFICEP)) {
			return CdmUtils.Nz(name.getInfraSpecificEpithet());
        }
        
		// Appended phrase        
        if (id.equals(P_ID_APPENDEDPHRASE)) {
        	return CdmUtils.Nz(name.getAppendedPhrase());
        }
        
		// Authorship cache        
        if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			return CdmUtils.Nz(name.getAuthorshipCache());
        }
        
		// Homotypical group        
        if (id.equals(P_ID_HOMOTYPICALGROUP)) {
			return CdmUtils.Nz(name.getHomotypicalGroup().getUuid().toString());
        }
        
		// Annotations, listed in custom property descriptor        
        if (id.equals(P_ID_ANNOTATIONS)) {
			return new AnnotationPropertySource(name.getAnnotations());
        }
        
		// Nomenclatural status        
        if (id.equals(P_ID_NOMSTATUS)) {
//			return new NomStatusPropertySource(name.getStatus());
//        	if (this.name.getRank() == null) return 0;
        	Set<NomenclaturalStatus> nomStatusSet = this.name.getStatus();
        	
        	if (nomStatusSet == null || nomStatusSet.size() == 0) {
        		return 0;
        	}
        	// TODO for now, only showing first nom status - change!
        	NomenclaturalStatus nomStatus = (NomenclaturalStatus) nomStatusSet.toArray()[0];
        	for (int i = 0; i < nomStatusTypes.length; i++) {
        		if (nomStatusTypes[i] == null) continue;
        		if (nomStatus.getType().getUuid().equals(nomStatusTypes[i].getUuid())) 
        			return i;
        	}
        	return 0;			
        }
        
		// Name relations, popup to edit list        
        if (id.equals(P_ID_NAMERELATIONS)) {
			return new NameRelationsPropertySource(name, name.getNameRelations());
        }
        
		// Type
        if (id.equals(P_ID_TYPE)) {
			return "";
        }
        
		// UUID        
        if (id.equals(P_ID_UUID)) {
			return CdmUtils.Nz(name.getUuid().toString());
        }
        
		// Markers        
        if (id.equals(P_ID_MARKERS)) {
			return new MarkersPropertySource(name.getMarkers());
        }

		// Taxon bases referring to this name        
        if (id.equals(P_ID_NUM_OF_BASES)) {
			return name.getTaxonBases().size();
        }
        
		// Object created by        
        if (id.equals(P_ID_CREATED)) {
        	if (name.getCreated() == null) {
        		return "";
        	} else {
        		return formatDate(name.getCreated().toDate());
        	}
        }
        
		// Object created when        
        if (id.equals(P_ID_CREATEDBY)) {
        	if (name.getCreatedBy() == null) {
        		return "";
        	} else {
        		return CdmUtils.Nz(name.getCreatedBy().generateTitle());
        	}
        }
        
		// Nomenclatural code used by parser        
        if (id.equals(P_ID_NOMENCLATURAL_CODE)) {
        	// Only shown if extending classes don't handle P_ID_NOMENCLATURAL_CODE 
        	return "none (nonviral name)";
        }
        
		// Nomenclatural reference        
        if (id.equals(P_ID_NOMENCLATURAL_REF)) {
        	
        	ReferenceBase nomRef = (ReferenceBase) name.getNomenclaturalReference();
			
			// Create nom. reference as necessary
			if (nomRef == null) {
				nomRef = Generic.NewInstance();
			}
			
			// Create property source for submenu
			ReferencePropertySource nomRefPropertySource = new NomenclaturalReferencePropertySource(nomRef);
			
			// Add listener to notify name of all changes to nom. reference
			nomRefPropertySource.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (evt.getNewValue() instanceof INomenclaturalReference) {
						name.setNomenclaturalReference((INomenclaturalReference) evt.getNewValue());
					}
				}
			});
			return nomRefPropertySource;
        	
        }
        
		// Microreference        
        if (id.equals(P_ID_NOMENCLATURAL_MICROREF)) {
			return CdmUtils.Nz(name.getNomenclaturalMicroReference());
        }
        return null;
	}
 
	/**
	 * The fields displayed under the title cache are dependent on rank.
	 * 
	 * @return
	 */
	private String[] getNameFields() {
		
		// An unparsed name has no name fields, only a cache. 
		//	Therefore, if rank is null, the name is unparsed.
		if (name.getRank() == null) {
			return null;
		}
		
    	List<String> nameFields = new ArrayList<String>();
    	nameFields.add(P_ID_UNINOMIAL);
    	if (name.isSupraGeneric() || name.isGenus()) { // Rank is higher than GENUS or equals GENUS
    	}
    	else if (name.isInfraGeneric()) { // lower than GENUS and higher than SPECIES
    		nameFields.add(P_ID_INFRAGENERICEP);
    	}   
    	else if (name.isSpecies()) { // Rank equals SPECIES
    		nameFields.add(P_ID_SPECIESEP);
    	}
    	else if (name.isInfraSpecific()) { // Rank is lower than SPECIES
    		nameFields.add(P_ID_SPECIESEP);
    		nameFields.add(P_ID_INFRASPECIFICEP);
    	} else {
    		nameFields.add(P_ID_SPECIESEP);
    	}
    	nameFields.add(P_ID_AUTHORSHIPCACHE);
    	nameFields.add(P_ID_APPENDEDPHRASE);
    	return nameFields.toArray(new String[nameFields.size()]);
	}
	
	private Object formatDate(Date date) {
		return String.format("%1$tm-%1$td-%1$tY %1$tH:%1$tM:%1$tS", date);
	}
	
	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {}

	/** 
	 * Any editable fields are set in the object here.
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
				
		// Since this is only called when the property value changes,
		//	it is a reliable place to fire the name's property change
		//	listener.
		name.firePropertyChange(ITaxEditorConstants.PROPERTY_SHEET_CHANGE, null, null);
		
		// Rank menu		
		if (id.equals(P_ID_RANK)) {
			int index = ((Integer) value).intValue();
			name.setRank(ranks[index]);
        }
		
		// Uninomial (aka Genus)		
		if (id.equals(P_ID_UNINOMIAL)) {
			name.setGenusOrUninomial((String) value);
		}
		
		// Infrageneric epithet		
        if (id.equals(P_ID_INFRAGENERICEP)) {
			name.setInfraGenericEpithet((String) value);
        }
        
		// Specific epithet        
        if (id.equals(P_ID_SPECIESEP)) {
			name.setSpecificEpithet((String) value);
        }
        
		// Infraspecific epithet        
        if (id.equals(P_ID_INFRASPECIFICEP)) {
			name.setInfraSpecificEpithet((String) value);
        }
        
		// Appended phrase        
        if (id.equals(P_ID_APPENDEDPHRASE)) {
			name.setAppendedPhrase((String) value);
        }
        
		// Authorship cache        
        if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			name.setAuthorshipCache((String) value);
        }
        
		// Annotations        
        if (id.equals(P_ID_ANNOTATIONS)) {
        	if (value instanceof Set) {
        		
        		Set existingAnnotations = name.getAnnotations();	
            	Set newAnnotations = (Set) value;
        		
        		for (Object object : newAnnotations) {
        			if (object instanceof Annotation && !existingAnnotations.contains(object)) {
        				name.addAnnotation((Annotation) object);
        			}
        		}
        		
        		for (Object object : existingAnnotations) {
        			if (object instanceof Annotation && !newAnnotations.contains(object)) {
        				name.removeAnnotation((Annotation) object);
        			}        			
        		}
        		
        	}
        }
		// Nomenclatural status        
        if (id.equals(P_ID_NOMSTATUS)) {
        	NomenclaturalStatus nomStatus;
			int index = ((Integer) value).intValue();
			Set<NomenclaturalStatus> nomStatusSet = name.getStatus();		

			if (nomStatusSet.size() > 0) {
				nomStatus =	(NomenclaturalStatus) nomStatusSet.toArray()[0];
				
				if (index == 0) {
					nomStatusSet.remove(nomStatus);
				} else {
					nomStatus.setType(nomStatusTypes[index]);
				}
				
			} else {
				nomStatus = NomenclaturalStatus.NewInstance(nomStatusTypes[index]);
				name.addStatus(nomStatus);
			}
        }
        
		// Markers        
        if (id.equals(P_ID_MARKERS)) {
        	// TODO set markers
        }
        
		// Nomenclatural reference        
        if (id.equals(P_ID_NOMENCLATURAL_REF)) {
        	if (value instanceof INomenclaturalReference) {
        		name.setNomenclaturalReference((INomenclaturalReference) value);
        	}
        }
        
		// Microreference        
        if (id.equals(P_ID_NOMENCLATURAL_MICROREF)) {
        	name.setNomenclaturalMicroReference((String) value);
        }
        
		// Reset both caches to reflect property sheet changes
		name.setTitleCache(name.generateTitle(), false);
		name.setFullTitleCache(name.generateFullTitle(), false);
		
		name.firePropertyChange(ITaxEditorConstants.REFRESH_NAMEVIEWER, null, null);
		
	}

	/** 
	 * Any node with children must override {@link java.lang.Object#toString()} 
	 * to display its name correctly
	 *  
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (parentid == null || parentid.equals("")) {
			return "";
		}
		if (parentid.equals(P_ID_TITLECACHE)) {
			return name.getTitleCache(); 
		}
		if (parentid.equals(P_ID_NOMENCLATURAL_REF)) {
        	INomenclaturalReference nomenclaturalReference = name.getNomenclaturalReference();
        	if (nomenclaturalReference == null) {
        		return "";
        	}        	
        	String microReference = name.getNomenclaturalMicroReference();
			return CdmUtils.Nz(nomenclaturalReference.getNomenclaturalCitation(microReference));
		}
		return super.toString();
	}
	
	public void addPropertyChangeListener(
			PropertyChangeListener listener) {
		this.propertyChangeSupport.addPropertyChangeListener(listener);
	}
}